/// <reference types="node" />
export declare type Step = {
    type: 'Step';
    text: string;
    argument?: {
        type: 'DocString';
        content: string;
    };
};
export declare type Scenario = {
    type: 'Background' | 'Scenario' | 'ScenarioOutline';
    name: string;
    steps: Step[];
    argument: string;
    keyword: string;
    examples?: Example[];
};
export declare type Feature = {
    type: 'Feature';
    name: string;
    children: Scenario[];
    tags: {
        type: 'Tag';
        name: string;
    }[];
};
export declare type Example = {
    type: 'Example';
    keyword: string;
    name: string;
    description?: string;
    tableHeader: TableRow;
    tableBody: TableRow[];
};
export declare type TableRow = {
    type: 'TableRow';
    cells: Cell[];
};
export declare type Cell = {
    type: 'TableCell';
    value: string;
};
export declare type SkippableFeature = Feature & {
    skip: boolean;
    dependsOn: Feature[];
};
export declare const parseFeatures: (featureData: Buffer[]) => SkippableFeature[];
export declare const fromDirectory: (dir: string) => Promise<SkippableFeature[]>;
