"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const runner_1 = require("./runner");
const Gherkin = require('gherkin');
const toposort = require('toposort');
const globAsync = require("glob");
const util_1 = require("util");
const path = require("path");
const glob = util_1.promisify(globAsync);
const fs_1 = require("fs");
const parser = new Gherkin.Parser(new Gherkin.AstBuilder());
const matcher = new Gherkin.TokenMatcher();
exports.parseFeatures = (featureData) => {
    const parsedFeatures = featureData.map(d => {
        const scanner = new Gherkin.TokenScanner(d.toString());
        return parser.parse(scanner, matcher).feature;
    });
    const sortedByLast = parsedFeatures.sort(({ tags: t1 }) => t1.find(({ name }) => name === '@Last') ? 1 : -1);
    const featureNames = sortedByLast.map(({ name }) => name);
    const featureDependencies = sortedByLast.map(feature => {
        const bg = feature.children.find(({ type }) => type === 'Background');
        if (!bg) {
            return [[feature.name, false]];
        }
        return bg.steps
            .filter(({ text }) => runner_1.afterRx.test(text))
            .reduce((deps, afterStep) => {
            const m = runner_1.afterRx.exec(afterStep.text);
            if (!m) {
                throw new Error(`Failed to find feature in ${afterStep.text}`);
            }
            if (!featureNames.includes(m[1])) {
                throw new Error(`The feature ${m[1]} you want to run after does not exist!`);
            }
            return [...deps, [m[1], feature.name]];
        }, []);
    });
    const sortedFeatureNames = toposort(featureDependencies.flat()).filter((feature) => feature);
    const dependencies = (f) => sortedFeatures.filter(({ name }) => {
        const depNames = featureDependencies
            .flat()
            .filter(([, fname]) => fname === f.name)
            .map(([depName]) => depName);
        return depNames.includes(name);
    });
    const sortedFeatures = sortedFeatureNames.map((featureName) => parsedFeatures.find(({ name }) => name === featureName));
    const isOnly = (f) => f.tags.find(({ name }) => name === '@Only');
    const only = parsedFeatures.filter(isOnly);
    const onlyNames = only.map(({ name }) => name);
    return sortedFeatures.map(f => {
        const { tags, name: featureName } = f;
        const skip = tags.find(({ name }) => name === '@Skip') ||
            (onlyNames.length && !onlyNames.includes(featureName));
        return {
            ...f,
            skip: !!skip,
            dependsOn: dependencies(f),
        };
    });
};
exports.fromDirectory = async (dir) => {
    const scan = path.join(path.resolve(dir), '*.feature');
    const featureFiles = await glob(scan);
    return exports.parseFeatures(featureFiles.map(f => fs_1.readFileSync(f)));
};
