/// <reference types="chai" />
import { Scenario, SkippableFeature, Step } from './load-features';
export declare type FlightRecorder = {
    flags: {
        [key: string]: boolean;
    };
    settings: {
        [key: string]: any;
    };
};
export declare const afterRx: RegExp;
export declare type Cleaner = <W extends Store>(runner: FeatureRunner<W>) => Promise<any>;
export declare class FeatureRunner<W> {
    private readonly featuresDir;
    private readonly reporters;
    private readonly stepRunners;
    private readonly cleaners;
    private readonly retry;
    store: Store;
    world: Store;
    constructor(world: W, options: {
        dir: string;
        reporters?: Reporter[];
        store?: Store;
        retry?: boolean;
    });
    addStepRunners(runners: StepRunner<W>[]): FeatureRunner<W>;
    cleanup(fn: Cleaner): void;
    progress(type: string, info?: string): Promise<void>;
    run(): Promise<RunResult>;
    runFeature(feature: SkippableFeature): Promise<FeatureResult>;
    retryScenario(scenario: Scenario, feature: FlightRecorder): Promise<ScenarioResult>;
    runScenario(scenario: Scenario, feature: FlightRecorder): Promise<ScenarioResult>;
    runStep(step: Step, feature: FlightRecorder): Promise<StepResult>;
    private replaceStoragePlaceholders;
}
export declare type Reporter = {
    report(result: RunResult): Promise<void>;
    progress(type: string, info?: string): Promise<void>;
};
export declare type StepResult = Result & {
    step: InterpolatedStep;
    skipped: boolean;
    result?: any;
    error?: Error | Chai.AssertionError;
};
export declare type ScenarioResult = Result & {
    scenario: Scenario;
    stepResults: StepResult[];
    tries: number;
    skipped: boolean;
};
export declare type FeatureResult = Result & {
    feature: SkippableFeature;
    scenarioResults: ScenarioResult[];
};
export declare type RunResult = Result & {
    featureResults: FeatureResult[];
};
export declare type Result = {
    success: boolean;
    runTime?: number;
    error?: Error;
};
export declare type StepRunner<W extends Store> = (step: InterpolatedStep) => false | StepRunnerFunc<W>;
export declare type StepRunnerFunc<W extends Store> = (runner: FeatureRunner<W>, feature: FlightRecorder) => Promise<any>;
export declare type InterpolatedStep = Step & {
    interpolatedText: string;
    interpolatedArgument?: string;
};
export declare class StepRunnerNotDefinedError extends Error {
    step: InterpolatedStep;
    constructor(step: InterpolatedStep);
}
export declare class RetryError extends Error {
    step: InterpolatedStep;
    constructor(step: InterpolatedStep);
}
export declare class StoreKeyUndefinedError extends Error {
    store: Store;
    keys: string[];
    constructor(keys: string[], store: Store);
}
export declare type Store = {
    [key: string]: any;
};
